@@ -22,7 +22,7 @@ use std::{
 use rand::{CryptoRng, Rng, RngCore};
 
 use snarkvm_algorithms::{SNARKError, SNARK};
-use snarkvm_fields::{PrimeField, ToConstraintField};
+use snarkvm_fields::{PoseidonMDSField, PrimeField, ToConstraintField};
 use snarkvm_gadgets::{
     bits::Boolean,
     nonnative::NonNativeFieldInputVar,
@@ -74,11 +74,12 @@ impl Debug for MarlinBound {
 /// The Marlin proof system.
 pub struct MarlinSNARK<
     F: PrimeField,
-    FSF: PrimeField,
+    FSF: PrimeField + PoseidonMDSField,
     PC: PolynomialCommitment<F>,
     FS: FiatShamirRng<F, FSF>,
     MC: MarlinMode,
     C: ConstraintSynthesizer<F>,
+    V: ToConstraintField<F>,
 > {
     f_phantom: PhantomData<F>,
     fsf_phantom: PhantomData<FSF>,
@@ -86,18 +87,20 @@ pub struct MarlinSNARK<
     fs_phantom: PhantomData<FS>,
     mc_phantom: PhantomData<MC>,
     c_phantom: PhantomData<C>,
+    v_phantom: PhantomData<V>,
 }
 
-impl<TargetField, BaseField, PC, FS, MM, C> MarlinSNARK<TargetField, BaseField, PC, FS, MM, C>
+impl<TargetField, BaseField, PC, FS, MM, C, V> MarlinSNARK<TargetField, BaseField, PC, FS, MM, C, V>
 where
     TargetField: PrimeField,
-    BaseField: PrimeField,
+    BaseField: PrimeField + PoseidonMDSField,
     PC: PolynomialCommitment<TargetField>,
     FS: FiatShamirRng<TargetField, BaseField>,
     MM: MarlinMode,
     PC::VerifierKey: ToConstraintField<BaseField>,
     PC::Commitment: ToConstraintField<BaseField>,
     C: ConstraintSynthesizer<TargetField>,
+    V: ToConstraintField<TargetField>,
 {
     /// Generates the universal proving and verifying keys for the argument system.
     pub fn universal_setup<R: Rng>(
@@ -157,23 +160,24 @@ where
     }
 }
 
-impl<TargetField, BaseField, PC, FS, MM, C> SNARK for MarlinSNARK<TargetField, BaseField, PC, FS, MM, C>
+impl<TargetField, BaseField, PC, FS, MM, C, V> SNARK for MarlinSNARK<TargetField, BaseField, PC, FS, MM, C, V>
 where
     TargetField: PrimeField,
-    BaseField: PrimeField,
+    BaseField: PrimeField + PoseidonMDSField,
     PC: PolynomialCommitment<TargetField>,
     FS: FiatShamirRng<TargetField, BaseField>,
     MM: MarlinMode,
     PC::VerifierKey: ToConstraintField<BaseField>,
     PC::Commitment: ToConstraintField<BaseField>,
     C: ConstraintSynthesizer<TargetField>,
+    V: ToConstraintField<TargetField>,
 {
     type AllocatedCircuit = C;
     type Circuit = (C, UniversalSRS<TargetField, PC>);
     type PreparedVerifyingKey = PreparedCircuitVerifyingKey<TargetField, PC>;
     type Proof = Proof<TargetField, PC>;
     type ProvingKey = CircuitProvingKey<TargetField, PC>;
-    type VerifierInput = [TargetField];
+    type VerifierInput = V;
     type VerifyingKey = CircuitVerifyingKey<TargetField, PC>;
 
     fn setup<R: RngCore>(
@@ -202,7 +206,11 @@ where
         input: &Self::VerifierInput,
         proof: &Self::Proof,
     ) -> Result<bool, SNARKError> {
-        match MarlinCore::<TargetField, BaseField, PC, FS, MM>::prepared_verify(verifying_key, input, proof) {
+        match MarlinCore::<TargetField, BaseField, PC, FS, MM>::prepared_verify(
+            verifying_key,
+            &input.to_field_elements()?,
+            proof,
+        ) {
             Ok(res) => Ok(res),
             Err(e) => Err(SNARKError::from(e)),
         }
@@ -213,7 +221,7 @@ where
 pub struct MarlinSNARKGadget<F, FSF, PC, FS, MM, PCG, FSG>
 where
     F: PrimeField,
-    FSF: PrimeField,
+    FSF: PrimeField + PoseidonMDSField,
     PC: PolynomialCommitment<F>,
     FS: FiatShamirRng<F, FSF>,
     MM: MarlinMode,
@@ -229,12 +237,12 @@ where
     fsg_phantom: PhantomData<FSG>,
 }
 
-impl<TargetField, BaseField, PC, FS, MM, PCG, FSG, C>
-    SNARKGadget<TargetField, BaseField, MarlinSNARK<TargetField, BaseField, PC, FS, MM, C>>
+impl<TargetField, BaseField, PC, FS, MM, PCG, FSG, C, V>
+    SNARKGadget<TargetField, BaseField, MarlinSNARK<TargetField, BaseField, PC, FS, MM, C, V>>
     for MarlinSNARKGadget<TargetField, BaseField, PC, FS, MM, PCG, FSG>
 where
     TargetField: PrimeField,
-    BaseField: PrimeField,
+    BaseField: PrimeField + PoseidonMDSField,
     PC: PolynomialCommitment<TargetField>,
     FS: FiatShamirRng<TargetField, BaseField>,
     MM: MarlinMode,
@@ -245,6 +253,7 @@ where
     PCG::VerifierKeyVar: ToConstraintFieldGadget<BaseField>,
     PCG::CommitmentVar: ToConstraintFieldGadget<BaseField>,
     C: ConstraintSynthesizer<TargetField>,
+    V: ToConstraintField<TargetField>,
 {
     type InputVar = NonNativeFieldInputVar<TargetField, BaseField>;
     type PreparedVerifyingKeyVar = PreparedCircuitVerifyingKeyVar<TargetField, BaseField, PC, PCG, FS, FSG>;
@@ -253,7 +262,7 @@ where
     type VerifyingKeyVar = CircuitVerifyingKeyVar<TargetField, BaseField, PC, PCG>;
 
     fn verifier_size(
-        circuit_vk: &<MarlinSNARK<TargetField, BaseField, PC, FS, MM, C> as SNARK>::VerifyingKey,
+        circuit_vk: &<MarlinSNARK<TargetField, BaseField, PC, FS, MM, C, V> as SNARK>::VerifyingKey,
     ) -> Self::VerifierSize {
         circuit_vk.circuit_info.num_variables
     }
@@ -428,7 +437,7 @@ pub mod test {
     type FS = FiatShamirAlgebraicSpongeRng<Fr, Fq, PoseidonSponge<Fq>>;
     type FSG = FiatShamirAlgebraicSpongeRngVar<Fr, Fq, PoseidonSponge<Fq>, PoseidonSpongeVar<Fq>>;
 
-    type TestSNARK = MarlinSNARK<Fr, Fq, PC, FS, MarlinRecursiveMode, Circuit<Fr>>;
+    type TestSNARK = MarlinSNARK<Fr, Fq, PC, FS, MarlinRecursiveMode, Circuit<Fr>, Vec<Fr>>;
     type TestSNARKGadget = MarlinSNARKGadget<Fr, Fq, PC, FS, MarlinRecursiveMode, PCGadget, FSG>;
 
     #[test]
@@ -459,7 +468,7 @@ pub mod test {
             let proof = TestSNARK::prove(&pk, &circ, &mut rng).unwrap();
 
             assert!(
-                TestSNARK::verify(&vk.clone().into(), &[c], &proof).unwrap(),
+                TestSNARK::verify(&vk.clone().into(), &vec![c], &proof).unwrap(),
                 "The native verification check fails."
             );
 
@@ -543,7 +552,7 @@ pub mod test {
         let proof = TestSNARK::prove(&pk, &circ, &mut rng).unwrap();
 
         assert!(
-            TestSNARK::verify(&vk.clone().into(), &[c], &proof).unwrap(),
+            TestSNARK::verify(&vk.clone().into(), &vec![c], &proof).unwrap(),
             "The native verification check fails."
         );
 
@@ -603,3 +612,368 @@ pub mod test {
         assert_eq!(verifier_gadget_constraints, VERIFIER_GADGET_CONSTRAINTS);
     }
 }
+
+#[cfg(test)]
+pub mod multiple_input_tests {
+    use core::ops::MulAssign;
+
+    use snarkvm_curves::{
+        bls12_377::{Bls12_377, Fq, Fr},
+        bw6_761::BW6_761,
+    };
+    use snarkvm_fields::Field;
+    use snarkvm_gadgets::{
+        curves::bls12_377::PairingGadget as Bls12_377PairingGadget,
+        traits::{alloc::AllocGadget, eq::EqGadget},
+    };
+    use snarkvm_polycommit::marlin_pc::{marlin_kzg10::MarlinKZG10Gadget, MarlinKZG10};
+    use snarkvm_r1cs::TestConstraintSystem;
+    use snarkvm_utilities::{test_rng, UniformRand};
+
+    use crate::{
+        constraints::snark::{MarlinSNARK, MarlinSNARKGadget},
+        fiat_shamir::{
+            FiatShamirAlgebraicSpongeRng,
+            FiatShamirAlgebraicSpongeRngVar,
+            PoseidonSponge,
+            PoseidonSpongeVar,
+        },
+        marlin::MarlinRecursiveMode,
+    };
+
+    use super::*;
+
+    const ITERATIONS: usize = 10;
+
+    #[derive(Copy, Clone)]
+    pub struct Circuit<F: Field> {
+        pub a: Option<F>,
+        pub b: Option<F>,
+        pub num_constraints: usize,
+        pub num_variables: usize,
+    }
+
+    impl<ConstraintF: Field> ConstraintSynthesizer<ConstraintF> for Circuit<ConstraintF> {
+        fn generate_constraints<CS: ConstraintSystem<ConstraintF>>(&self, cs: &mut CS) -> Result<(), SynthesisError> {
+            let a = cs.alloc(|| "a", || self.a.ok_or(SynthesisError::AssignmentMissing))?;
+            let b = cs.alloc(|| "b", || self.b.ok_or(SynthesisError::AssignmentMissing))?;
+            let c = cs.alloc_input(
+                || "c",
+                || {
+                    let mut a = self.a.ok_or(SynthesisError::AssignmentMissing)?;
+                    let b = self.b.ok_or(SynthesisError::AssignmentMissing)?;
+
+                    a.mul_assign(&b);
+                    Ok(a)
+                },
+            )?;
+
+            let d = cs.alloc_input(
+                || "d",
+                || {
+                    let mut a = self.a.ok_or(SynthesisError::AssignmentMissing)?;
+                    let b = self.b.ok_or(SynthesisError::AssignmentMissing)?;
+
+                    a.mul_assign(&b);
+                    Ok(a)
+                },
+            )?;
+
+            for i in 0..(self.num_variables - 3) {
+                let _ = cs.alloc(
+                    || format!("var {}", i),
+                    || self.a.ok_or(SynthesisError::AssignmentMissing),
+                )?;
+            }
+
+            for i in 0..(self.num_constraints - 1) {
+                cs.enforce(|| format!("constraint {}", i), |lc| lc + a, |lc| lc + b, |lc| lc + c);
+            }
+
+            for i in 0..(self.num_constraints - 1) {
+                cs.enforce(|| format!("constraint 2 {}", i), |lc| lc + a, |lc| lc + b, |lc| lc + d);
+            }
+
+            Ok(())
+        }
+    }
+
+    pub struct VerifierCircuit<
+        F: PrimeField,
+        ConstraintF: PrimeField + PoseidonMDSField,
+        PC: PolynomialCommitment<F>,
+        FS: FiatShamirRng<F, ConstraintF>,
+        MM: MarlinMode,
+        PCG: PCCheckVar<F, PC, ConstraintF>,
+        FSG: FiatShamirRngVar<F, ConstraintF, FS>,
+    >
+    where
+        PC::VerifierKey: ToConstraintField<ConstraintF>,
+        PC::Commitment: ToConstraintField<ConstraintF>,
+        PCG::VerifierKeyVar: ToConstraintFieldGadget<ConstraintF>,
+        PCG::CommitmentVar: ToConstraintFieldGadget<ConstraintF>,
+    {
+        pub c: F,
+        pub verifying_key: CircuitVerifyingKey<F, PC>,
+        pub proof: Proof<F, PC>,
+        _f: PhantomData<ConstraintF>,
+        _fs: PhantomData<FS>,
+        _marlin_mode: PhantomData<MM>,
+        _pcg: PhantomData<PCG>,
+        _fsg: PhantomData<FSG>,
+    }
+
+    impl<
+        F: PrimeField,
+        ConstraintF: PrimeField + PoseidonMDSField,
+        PC: PolynomialCommitment<F>,
+        FS: FiatShamirRng<F, ConstraintF>,
+        MM: MarlinMode,
+        PCG: PCCheckVar<F, PC, ConstraintF>,
+        FSG: FiatShamirRngVar<F, ConstraintF, FS>,
+    > ConstraintSynthesizer<ConstraintF> for VerifierCircuit<F, ConstraintF, PC, FS, MM, PCG, FSG>
+    where
+        PC::VerifierKey: ToConstraintField<ConstraintF>,
+        PC::Commitment: ToConstraintField<ConstraintF>,
+        PCG::VerifierKeyVar: ToConstraintFieldGadget<ConstraintF>,
+        PCG::CommitmentVar: ToConstraintFieldGadget<ConstraintF>,
+    {
+        fn generate_constraints<CS: ConstraintSystem<ConstraintF>>(&self, cs: &mut CS) -> Result<(), SynthesisError> {
+            let vk_gadget = CircuitVerifyingKeyVar::<F, ConstraintF, PC, PCG>::alloc(cs.ns(|| "vk"), || {
+                Ok(self.verifying_key.clone())
+            })?;
+
+            let proof_gadget =
+                ProofVar::<F, ConstraintF, PC, PCG>::alloc(cs.ns(|| "proof"), || Ok(self.proof.clone()))?;
+
+            let input_gadget = NonNativeFieldInputVar::<F, ConstraintF>::alloc(cs.ns(|| "input 2"), || {
+                Ok(vec![self.c.clone(), self.c.clone()])
+            })?;
+
+            let output = MarlinVerificationGadget::<F, ConstraintF, PC, PCG>::verify::<_, FS, FSG>(
+                cs.ns(|| "verify"),
+                &vk_gadget,
+                &input_gadget.val,
+                &proof_gadget,
+            )
+            .unwrap();
+
+            let expected = Boolean::Constant(true);
+
+            output.enforce_equal(cs.ns(|| "valid_verification"), &expected)?;
+
+            Ok(())
+        }
+    }
+
+    type PC = MarlinKZG10<Bls12_377>;
+    type PCGadget = MarlinKZG10Gadget<Bls12_377, BW6_761, Bls12_377PairingGadget>;
+
+    type FS = FiatShamirAlgebraicSpongeRng<Fr, Fq, PoseidonSponge<Fq>>;
+    type FSG = FiatShamirAlgebraicSpongeRngVar<Fr, Fq, PoseidonSponge<Fq>, PoseidonSpongeVar<Fq>>;
+
+    type TestSNARK = MarlinSNARK<Fr, Fq, PC, FS, MarlinRecursiveMode, Circuit<Fr>, Vec<Fr>>;
+    type TestSNARKGadget = MarlinSNARKGadget<Fr, Fq, PC, FS, MarlinRecursiveMode, PCGadget, FSG>;
+
+    #[test]
+    fn two_input_marlin_snark_test() {
+        let mut rng = test_rng();
+
+        for _ in 0..ITERATIONS {
+            // Construct the circuit.
+
+            let a = Fr::rand(&mut rng);
+            let b = Fr::rand(&mut rng);
+            let mut c = a;
+            c.mul_assign(&b);
+
+            let circ = Circuit {
+                a: Some(a),
+                b: Some(b),
+                num_constraints: 100,
+                num_variables: 25,
+            };
+
+            // Generate the circuit parameters.
+
+            let (pk, vk) = TestSNARK::circuit_specific_setup(circ, &mut rng).unwrap();
+
+            // Test native proof and verification.
+
+            let proof = TestSNARK::prove(&pk, &circ, &mut rng).unwrap();
+
+            assert!(
+                TestSNARK::verify(&vk.clone().into(), &[c.clone(), c].to_vec(), &proof).unwrap(),
+                "The native verification check fails."
+            );
+
+            // Initialize constraint system.
+            let mut cs = TestConstraintSystem::<Fq>::new();
+
+            let input_gadget = <TestSNARKGadget as SNARKGadget<Fr, Fq, TestSNARK>>::InputVar::alloc_input(
+                cs.ns(|| "alloc_input_gadget"),
+                || Ok(vec![c.clone(), c]),
+            )
+            .unwrap();
+
+            let proof_gadget =
+                <TestSNARKGadget as SNARKGadget<Fr, Fq, TestSNARK>>::ProofVar::alloc(cs.ns(|| "alloc_proof"), || {
+                    Ok(proof)
+                })
+                .unwrap();
+
+            let vk_gadget = <TestSNARKGadget as SNARKGadget<Fr, Fq, TestSNARK>>::VerifyingKeyVar::alloc(
+                cs.ns(|| "alloc_vk"),
+                || Ok(vk.clone()),
+            )
+            .unwrap();
+
+            assert!(
+                cs.is_satisfied(),
+                "Constraints not satisfied: {}",
+                cs.which_is_unsatisfied().unwrap()
+            );
+
+            let verification_result = <TestSNARKGadget as SNARKGadget<Fr, Fq, TestSNARK>>::verify(
+                cs.ns(|| "marlin_verify"),
+                &vk_gadget,
+                &input_gadget,
+                &proof_gadget,
+            )
+            .unwrap();
+
+            assert!(
+                cs.is_satisfied(),
+                "Constraints not satisfied: {}",
+                cs.which_is_unsatisfied().unwrap()
+            );
+
+            verification_result
+                .enforce_equal(cs.ns(|| "enforce_equal_verification"), &Boolean::Constant(true))
+                .unwrap();
+
+            assert!(
+                cs.is_satisfied(),
+                "Constraints not satisfied: {}",
+                cs.which_is_unsatisfied().unwrap()
+            );
+        }
+    }
+
+    #[test]
+    fn marlin_nested_verification_gadget_test() {
+        let mut rng = test_rng();
+
+        for _ in 0..ITERATIONS {
+            // Construct the circuit.
+
+            let a = Fr::rand(&mut rng);
+            let b = Fr::rand(&mut rng);
+            let mut c = a;
+            c.mul_assign(&b);
+
+            let circ = Circuit {
+                a: Some(a),
+                b: Some(b),
+                num_constraints: 100,
+                num_variables: 25,
+            };
+
+            // Generate the circuit parameters.
+
+            let (pk, vk) = TestSNARK::circuit_specific_setup(circ, &mut rng).unwrap();
+
+            // Test native proof and verification.
+
+            let proof = TestSNARK::prove(&pk, &circ, &mut rng).unwrap();
+
+            assert!(
+                TestSNARK::verify(&vk.clone().into(), &[c.clone(), c].to_vec(), &proof).unwrap(),
+                "The native verification check fails."
+            );
+
+            // Initialize constraint system.
+            let mut cs = TestConstraintSystem::<Fq>::new();
+
+            let circuit = VerifierCircuit::<Fr, Fq, PC, FS, MarlinRecursiveMode, PCGadget, FSG> {
+                c: c.clone(),
+                verifying_key: vk,
+                proof,
+                _f: PhantomData,
+                _fs: PhantomData,
+                _marlin_mode: PhantomData,
+                _pcg: PhantomData,
+                _fsg: PhantomData,
+            };
+
+            circuit
+                .generate_constraints(&mut cs.ns(|| "verify_within_gadget"))
+                .unwrap();
+
+            assert!(
+                cs.is_satisfied(),
+                "Constraints not satisfied: {}",
+                cs.which_is_unsatisfied().unwrap()
+            );
+        }
+    }
+
+    #[test]
+    fn marlin_test_nested_snark() {
+        let mut rng = test_rng();
+
+        // Construct the circuit.
+
+        let a = Fr::rand(&mut rng);
+        let b = Fr::rand(&mut rng);
+        let mut c = a;
+        c.mul_assign(&b);
+
+        let circ = Circuit {
+            a: Some(a),
+            b: Some(b),
+            num_constraints: 100,
+            num_variables: 25,
+        };
+
+        // Generate the circuit parameters.
+
+        let (pk, vk) = TestSNARK::circuit_specific_setup(circ, &mut rng).unwrap();
+
+        // Test native proof and verification.
+
+        let proof = TestSNARK::prove(&pk, &circ, &mut rng).unwrap();
+
+        assert!(
+            TestSNARK::verify(&vk.clone().into(), &[c.clone(), c].to_vec(), &proof).unwrap(),
+            "The native verification check fails."
+        );
+
+        // Initialize constraint system.
+        let nested_circuit = VerifierCircuit::<Fr, Fq, PC, FS, MarlinRecursiveMode, PCGadget, FSG> {
+            c: c.clone(),
+            verifying_key: vk,
+            proof,
+            _f: PhantomData,
+            _fs: PhantomData,
+            _marlin_mode: PhantomData,
+            _pcg: PhantomData,
+            _fsg: PhantomData,
+        };
+
+        use snarkvm_algorithms::snark::groth16::Groth16;
+        type NestedSNARK =
+            Groth16<BW6_761, VerifierCircuit<Fr, Fq, PC, FS, MarlinRecursiveMode, PCGadget, FSG>, Vec<Fq>>;
+
+        let (nested_pk, nested_vk) = NestedSNARK::setup(&nested_circuit, &mut rng).unwrap();
+
+        // Test native proof and verification.
+
+        let nested_proof = NestedSNARK::prove(&nested_pk, &nested_circuit, &mut rng).unwrap();
+
+        assert!(
+            NestedSNARK::verify(&nested_vk.clone().into(), &vec![], &nested_proof).unwrap(),
+            "The native verification check fails."
+        );
+    }
+}
